package de.pheerai.rcdbquery.dataMappings.order

import de.pheerai.rcdbquery.dataMappings.internal.ParamKey
import de.pheerai.rcdbquery.dataMappings.internal.RcdbParamOption
import de.pheerai.rcdbquery.dataMappings.internal.RelevantForAll

@Suppress("unused")
public enum class Order(
    override val prettyName: String,
    override val fullName: String,
    override val paramValue: Long,
    override val relevantForAll: Boolean = true,
) : RcdbParamOption<Long>,
    RelevantForAll {
    IMAGES("Images", "Amount of Images", 0),
    NAME("Name", "Name of the Coaster", 1),
    AKA("AKA", "Is/was also known as", 2, relevantForAll = false),
    LOCATION("Location", 3),
    PARK("Park", 4),
    OPENED("Opened", "Opening Date", 8),
    CLOSED("Closed", "Closing Date (past)", 9),

    @Deprecated(
        "This field seems to have been deprecated in favor of another parameter",
        level = DeprecationLevel.WARNING,
        replaceWith = ReplaceWith("Order.STATE")
    )
    STATE_2("State", "Operational State (legacy)", 10, relevantForAll = false),
    ENTERED("Entered", 12),
    TYPE("Type", "Construction Type (Wood, Steel)", 13),
    DESIGN("Design", "Train Design (Sit-Down, Invert,...)", 14),
    CLASSIFICATION("Classification", "Coaster Classification (Coaster, Powered Coaster, Mountain Coaster)", 15),
    LAYOUT("Layout", 16),
    MODEL("Model", "Manufacturer Model", 17),
    MODEL_LINE("Model-Line", "Coaster Model Line", 18),
    SPEED("Speed", "Max. Speed", 19),
    HEIGHT("Height", 20),
    DROP("Drop", "Highest Drop", 21),
    LENGTH("Length", 22),
    ANGLE("Angle", 23),
    INVERSIONS("Inversions", "Number of inversions", 24),
    SERIAL_NO("Serial", "Serial number", 25, relevantForAll = false),
    SCALE("Scale", "Thrill factor", 26),
    RIDE_TIME("Ride Time", 27),
    STATUS("Status", "Operational state", 29),
    MANUFACTURER("Manufacturer", 30),
    CLOSING("Closing", "(Future) closign date for temporary installations", 31)
    ;

    constructor(name: String, paramId: Long) : this(name, name, paramId)

    public companion object : ParamKey {
        override val paramKey: String = "order"
    }
}
