package de.pheerai.rcdbquery.dsl

import de.pheerai.rcdbquery.dsl.params.MultiParams
import de.pheerai.rcdbquery.dsl.params.ParamsCollector
import de.pheerai.rcdbquery.dsl.params.SingleParams

public data class RcdbParams(
    private val multiParams: MultiParams,
    private val singleParams: SingleParams,
) {
    public fun toStrings(): List<String> {
        val multiParamSequence: Sequence<Pair<String, String>> = this.multiParams
            .params
            .mapValues { it.value.joinToString(separator = ",") }
            .map { it.toPair() }
            .asSequence()
        val singleParamSequence: Sequence<Pair<String, String>> = this.singleParams
            .params
            .asSequence()
            .map { it.toPair() }
        return singleParamSequence.plus(multiParamSequence)
            .map { p -> "${p.first}=${p.second}" }
            .toList()
    }
}

public fun rcdbQuery(body: ParamsCollector.() -> ParamsCollector): String =
    RcdbUrlQuery("https://www.rcdb.com/r.htm?", rcdbQueryParams(body)).toString()

private data class RcdbUrlQuery(val baseUrl: String, val params: RcdbParams) {
    override fun toString(): String {
        return params.toStrings()
            .joinToString(prefix = baseUrl, separator = "&")
    }
}

/**
 * Builder for the parameters only (mainly because it might be possible in the future to have an API with a POST search object?
 */
private fun rcdbQueryParams(body: ParamsCollector.() -> ParamsCollector): RcdbParams {
    val builder = ParamsCollector()
    builder.body()
    return builder.build()
}
